@@ -29,7 +29,7 @@ sys.path.append("/usr/lib/asterisk-phonepatch")
 import templateparser, dtmf, radio
 import radiocontrol, daemonize
 
-__version__ = "$Revision: 1.7 $"
+__version__ = "$Revision: 1.8 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
 __depends__ = ['Asterisk', 'Sox', 'Festival', 'Python-2.4']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
@@ -112,7 +112,7 @@ class Phonepatch:
 		# Configuration options
 		self.festival_indicator = "@"
 		
-		# Create signals dictionary: key = <signal_int> - value = <signal_name">
+		# Create signals dictionary: key = signal_int - value = signal_name
 		self.signals = {}
 		for var, value in inspect.getmembers(signal):
 			if var.find("SIG") == 0 and var.find("SIG_") != 0: 
@@ -174,7 +174,6 @@ class Phonepatch:
 			channels = self.asterisk_channels, \
 			sensibility = self.getconf("dtmf_sensibility"), \
 			verbose = False)
-		
 		self.pidfile = os.path.join(PIDFILE_DIR, self.phonepatch_phpconfig + ".pid")
 		self.language = self.getconf("language")
 		self.sounds_dir = self.getconf("sounds_dir")
@@ -214,7 +213,6 @@ class Phonepatch:
 			elif section != DEFAULT_SECTION:
 				self.phonepatch_extensions.append(section)
 
-
 	###############################
 	def set_state(self, state):
 		"""Set phonepatch state (incall/outcall/daemon)"""
@@ -334,7 +332,7 @@ class Phonepatch:
 		# Create radio instance (control soundcard and PTT)
 		try:self.radio = radio.Radio(self.getconf("soundcard_device"), self.asterisk_samplerate, \
 			self.ptt, self.carrier, verbose=self.modules_verbose, fullduplex = self.getconf("full_duplex"), \
-			soundcard_retries = 5, latency = self.getconf("soundcard_latency"), enable_ctcss=True)
+			soundcard_retries = 5, latency = self.getconf("soundcard_latency"), ctcss_decoder=self.getconf("ctcss_decoder_mintime"))
 		except IOError, detail:
 			self.debug("open_radio: %s" %str(detail), 1)
 			sys.exit(1)
@@ -452,7 +450,7 @@ class Phonepatch:
 			# It's the same, so using cached audio buffer
 			self.debug("play: using cached audio")
 		else:
-			# Load <args> (play_file() for audiofiles and play_text() for text-to-speech)
+			# Load args (play_file() for audiofiles and play_text() for text-to-speech)
 			self.raw_buffer = ""
 			for option in args.split(","):
 				if not option: continue
@@ -537,8 +535,8 @@ class Phonepatch:
 	def flush_asterisk(self, write=True):
 		if not self.interface: return True
 		try: 
-			retsel = select.select([self.asterisk_out], [], [], 0.2)[0]
-			if self.asterisk_out not in retsel: return False
+			retsel = select.select([self.asterisk_out], [], [], 0.1)[0]
+			if self.asterisk_out not in retsel: return True
 			buffer = self.secure_os(os.read, self.asterisk_out, self.buffer_size)
 			if not buffer: return False
 			if write:
@@ -645,7 +643,7 @@ class Phonepatch:
 		"""Create pidfile when a daemon process starts"""
 		self.debug("create_pidfile: %s" %self.pidfile)
 		try: fd = open(self.pidfile, "w")
-		except: self.debug("create_pidfile: pidfile could not be opened for writing", exit = 1, delete_pidfile = False)
+		except: self.debug("create_pidfile: pidfile could not be opened for writing", exit=1)
 		fd.write(str(os.getpid()) + "\n")
 		fd.close()
 		os.chown(self.pidfile, *self.get_asterisk_id())
@@ -696,7 +694,7 @@ class Phonepatch:
 
 	###################################
 	def process_incall(self):
-		"""Waits for DTMF <answer_button> (with a timeout) and open the interface if received"""
+		"""Waits for DTMF answer_button or CTCSS tone (with a timeout) and open the interface if received"""
 		incall = self.getconf("incall")
 		calltimeout = time.time() +self.getconf("incall_report_timeout")
 		answer_button = self.getconf("incall_answer_button")
@@ -725,7 +723,7 @@ class Phonepatch:
 					for key in keys: self.debug("process_incall: DTMF button received: %s" %key)
 					if answer_button in keys:				
 						return "answered"
-				elif "ctcss" in modes:
+				elif mode == "ctcss":
 					self.radio.decode_ctcss(buffer)
 					tone = self.radio.get_ctcss_tone()
 					if tone == ctcss_rx_freq:
@@ -772,7 +770,7 @@ class Phonepatch:
 			
 	###################################
 	def make_call(self, number, reopen=True):
-		"""Use outgoing calls Asterisk facility to call <number>"""
+		"""Use outgoing calls Asterisk facility to call number"""
 		number = self.process_number(str(number))
 		if not number: 
 			self.debug("make_call: process_number() not succesful")
@@ -813,7 +811,7 @@ class Phonepatch:
 		# Save outcall spool file name on class object, as callback continue_outcall() uses it
 		self.outcallfile = os.path.join(self.outcalls_dir, os.path.basename(callpath))
 		
-		# Loop until spool file is processed or a <sleep_signal> received
+		# Loop until spool file is processed or a sleep_signal received
 		# TODO: fullduplex
 		while 1:
 			if self.play(True, False, self.getconf("ring_audio"), max_time = self.getconf("ring_audio_time"), \
@@ -844,14 +842,14 @@ class Phonepatch:
 			self.debug("make_call: Asterisk was unable to connect")
 			return
 		
-		# Ok, we received the <sleep_signal>, time to sleep 
+		# Ok, we received the sleep_signal, time to sleep 
 		self.debug("make_call: sleep_signal received, an AGI-Phonepatch has been launched")
 		self.sleep_daemon(self.getconf("call_limit"), reopen)
 		return True
 		
 	###################################
 	def set_signals(self, signals):
-		"""Bind a list of signal to default <signal_handler>"""
+		"""Bind a list of signal to default signal_handler"""
 		for sig in signals:
 			signal.signal(sig, self.signal_handler)
 
@@ -947,7 +945,7 @@ class Phonepatch:
 
 	###################################
 	def loop_daemon(self):
-		# Wait for <asktone> button, record number and make a call when received <outcall_button>
+		# Wait for asktone button, record number and make a call when received outcall_button
 		mode = self.getconf("outcall_askfortone_mode")
 		button = self.getconf("askfortone_button")
 		if not mode: self.debug("loop_daemon: outcall_askfortone_mode not defined"); return
@@ -964,17 +962,15 @@ class Phonepatch:
 
 				if not self.flush_asterisk(): return
 				buffer = self.radio.read_audio(self.buffer_size)
-				self.radio.decode_ctcss(buffer)
 				if mode == "dtmf":
 					keys = self.dtmf_decoder.decode_buffer(buffer)
 					for key in keys: self.debug("loop_daemon: DTMF button received: %s" %key)
 					if button in keys:				
 						break
+				self.radio.decode_ctcss(buffer)
 				extension = self.check_ctcss()
 				if not extension: continue
-				mode = self.getconf("outcall_askfortone_mode", extension)
-				if mode != "ctcss":
-					self.debug("loop_daemon: extension_askfortone_mode different from 'ctcss' in phonepatch: %s" %extension)
+				if self.getconf("outcall_askfortone_mode", extension) != "ctcss":
 					continue
 				number = self.getconf("outcall_ctcss_autocall", extension)
 				if number:
@@ -993,7 +989,7 @@ class Phonepatch:
 			# AskForTone DTMF button received, now record the destination number
 			# TODO: Fullduplex
 			self.play(True, False, self.getconf("tone_audio"), max_time=self.getconf("tone_audio_time"), loop=True)
-			self.debug("loop_daemon: waiting for number and <outcall_button>")
+			self.debug("loop_daemon: waiting for number and outcall_buttony")
 			timeout_time = time.time() + self.getconf("tone_timeout")
 			dtmf_keys = []
 		
@@ -1042,13 +1038,18 @@ class Phonepatch:
 		"""Phonepatch acting as AGI"""
 		self.set_state("agicall")
 		self.get_agi_keys(sys.stdin)
-		#extension = self.agi_keys["extension"]
-		#if not extension:
-		#	self.debug("agicall: extension not found in AGI keys. Aborting"); return
 		if extension not in self.configuration:
 			self.debug("agicall: phonepatch extension not defined: %s" %extension, 1)
 		self.phonepatch_extension = extension
-		self.init_php(self.getconf("phonepatch"))
+		phonepatch = self.getconf("phonepatch")
+		if not phonepatch:
+			if not self.phonepatch_configs:
+				self.debug("agicall: no phonepatchs defined", 1)
+			phonepatch = self.phonepatch_configs[0]
+			self.debug("agicall: no phonepatchs defined, selecting first: %s" %phonepatch)
+		if not self.init_php(phonepatch): 
+			self.debug("agicall: error in phonepatch init", 1)
+			return
 		if self.agi_keys.get("accountcode") == OUTCALL_IDNAME: 
 			self.outcall()
 		else: self.incall()
@@ -1061,7 +1062,7 @@ class Phonepatch:
 		When an incall or outcall start, this process will be stopped by a signal
 		"""
 		self.background = background
-		self.init_php(phonepatch)
+		if not self.init_php(phonepatch): return
 		if not self.getconf("outcall_daemon"):
 			if not background:
 				self.debug("daemon: outcall_daemon disabled, daemon not loaded for phonepatch: %s" %phonepatch)
@@ -1115,7 +1116,7 @@ class Phonepatch:
 		self.destination =  self.getconf("username")
 		self.set_signals([signal.SIGHUP])
 		
-		# Send a <stop_signal> to daemon process (if running)
+		# Send a stop_signal to daemon process (if running)
 		self.signal_daemon(signal.SIGUSR1)
 		self.open_radio()
 		self.open_interface(self.asterisk_fifo)
@@ -1175,7 +1176,6 @@ and spawn outgoing calls when required (radio DTMF controlled)"""
 			phonepatchs = get_phpconfigs(configuration)
 			if not phonepatchs: sys.stderr.write("no phonepatchs found in configuration\n"); sys.exit(1)
 			if not options.background: 
-				phonepatchs.sort()
 				phonepatchs = [phonepatchs[0]]
 				sys.stdout.write("using default phonepatch: %s\n" %phonepatchs[0])
 		for phpname in phonepatchs:
