@@ -26,7 +26,7 @@ import struct
 # External phonepatch modules
 import soundcard, ctcss
 
-__version__ = "$Revision: 1.6 $"
+__version__ = "$Revision: 1.7 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
 __depends__ = ['Python-2.4']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
@@ -44,7 +44,8 @@ class Radio:
 	"""
 
 	###############################
-	def __init__(self, soundcard_device, samplerate, ptt, carrier, verbose=False, soundcard_retries=1, fullduplex=False, latency=None, enable_ctcss=None):
+	def __init__(self, soundcard_device, samplerate, ptt, carrier, verbose=False, \
+		soundcard_retries=1, fullduplex=False, latency=None, ctcss_decoder=None):
 		"""Open a soundcard and PTT interface.
 
 		Use radio_control object to set PTT and get carrier-detection state.
@@ -86,9 +87,9 @@ class Radio:
 		self.set_carrier_state(False)
 	
 		# CTCSS generator/decoder		
-		if enable_ctcss:
+		if ctcss_decoder:
 			self.ctcss_generator = ctcss.Generator(self.samplerate, self.sample_width)
-			self.ctcss_decoder = ctcss.Decoder(self.samplerate, self.sample_width, 1.0)
+			self.ctcss_decoder = ctcss.Decoder(self.samplerate, self.sample_width, ctcss_decoder)
 		
 		# Open soundcard
 		self.soundcard = None
